#!/usr/bin/env python
# -*- coding: utf-8 -*-

PKG_NAME = 'rospeex_core'

try:
    import roslib; roslib.load_manifest(PKG_NAME)
except:
    pass

import os
import rospy
import logging
import unittest
from nose.tools import eq_, raises, nottest
import ConfigParser


from rospeex_core.ss.client_voicetext import SpeechSynthesisClient_VoiceText
from rospeex_core.exceptions import ParameterException
from rospeex_core.exceptions import InvalidResponseException
from rospeex_core.exceptions import InvalidRequestException
from rospeex_core.exceptions import RequestTimeoutException
from rospeex_core.exceptions import SpeechSynthesisException
from rospeex_core.exceptions import UnsupportedLanguageException


# set logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s:%(levelname)s:%(filename)s:%(lineno)d:%(message)s')
ch = logging.StreamHandler()
ch.setFormatter(formatter)
logger.addHandler(ch)


class TestSpeechSynthesisClient_VoiceText(unittest.TestCase):
    def setUp(self):
        rospy.init_node(PKG_NAME)

        base_dir = os.path.dirname(__file__)
        settings = ConfigParser.ConfigParser()
        filename = os.path.join(base_dir, 'config.cfg')

        settings.read(filename)

        self.big_text_file = os.path.join(base_dir, settings.get('SpeechSynthesis', 'big_text_for_voicetext_file'))
        self.api_key = settings.get('SpeechSynthesis', 'voicetext_api_key')

        self.big_text_data = None
        with open(self.big_text_file, 'r') as f:
            self.big_text_data = f.read()


    @raises(ParameterException)
    def test_request_invalid_message(self):
        message    = None
        language   = 'ja'
        voice_font = '*'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_too_long_message(self):
        message    = ''
        language   = 'ja'
        voice_font = 'haruka'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key

        # create too long message
        for i in range(300):
            message = message + 'a'

        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_long_utf8_message(self):
        message    = u''
        language   = 'ja'
        voice_font = 'haruka'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key

        # create too long message
        for i in range(201):
            message = message + u'あ'
        client.request(message, language, voice_font, timeout)


    @nottest
    @raises(InvalidResponseException)
    def test_request_invalid_message_charactor(self):
        message    = '<'
        language   = 'ja'
        voice_font = 'haruka'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client.request(message, language, voice_font, timeout)


    @raises(UnsupportedLanguageException)
    def test_request_invalid_language(self):
        message    = 'hello'
        language   = 'de'        # deutsch
        voice_font = 'haruka'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_api_key(self):
        message    = 'hello'
        language   = 'ja'        # deutsch
        voice_font = 'haruka'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client.request(message, language, voice_font, timeout)


    @raises(InvalidResponseException)
    def test_request_invalid_url(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'haruka'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client.URL = 'http://rospeex.ucri.jgn-x.jp/nauth_json/jsServices/hoge'
        client._key = self.api_key
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_voice_font(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'hogehoge'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_pitch_high(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'show'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client._pitch = 201
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_pitch_low(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'show'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client._pitch = 49
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_volume_high(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'show'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client._volume = 201
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_volume_low(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'show'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client._volume = 49
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_speed_high(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'show'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client._speed = 401
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_speed_low(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'show'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client._speed = 49
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_emotional_user(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'show'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client._emotion = 'anger'
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_emotion_type(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'bear'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client._emotion = 'hoge'
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_emotion_type(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'bear'
        timeout    = 100
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client._emotion = 'sadness'
        client._emotion_level = 3
        client.request(message, language, voice_font, timeout)


    @nottest
    @raises(RequestTimeoutException)
    def test_request_server_timeout_post(self):
        message    = self.big_text_data
        language   = 'ja'
        voice_font = 'haruka'
        timeout    = 2
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client.request(message, language, voice_font, timeout)


    def test_request_valid_big_message(self):
        message    = self.big_text_data        # alice in wonder land
        language   = 'ja'
        voice_font = 'haruka'
        timeout    = 1000000
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client.request(message, language, voice_font, timeout)


    def test_request_valid_japanese_message(self):
        message    = u'こんにちは'
        language   = 'ja'
        voice_font = 'haruka'
        timeout    = 10000
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key
        client.request(message, language, voice_font, timeout)


    def test_request_valid_voice_fonts(self):
        message    = u'こんにちは'
        language   = 'ja'
        timeout    = 10000
        client = SpeechSynthesisClient_VoiceText()
        client._key = self.api_key

        for speaker in client.SPEAKER_LIST:
            client.request(message, language, speaker, timeout)



if __name__ == '__main__':
    import rosunit
    test_class = TestSpeechSynthesisClient_VoiceText
    rosunit.unitrun(PKG_NAME, 'speech_synthesis_client_voicetext', test_class, None, coverage_packages=['rospeex.ss'])
